import 'dart:convert';

import 'package:http/http.dart';
import 'phrase_exceptions.dart';

class PhraseApiResult {
  PhraseApiResult._();
  factory PhraseApiResult.success(String version, String arb) = PhraseApiResultSuccess;
  factory PhraseApiResult.notModified() = PhraseApiResultNotModified;
}
class PhraseApiResultSuccess extends PhraseApiResult {
  PhraseApiResultSuccess(this.version, this.arb): super._();
  final String version;
  final String arb;
}
class PhraseApiResultNotModified extends PhraseApiResult {
  PhraseApiResultNotModified(): super._();
}

class PhraseApi {
  static const String _BASE_URL = 'ota.phraseapp.com';

  Future<StreamedResponse> _get(Uri uri){
    Request req = Request("GET", uri)..followRedirects = false;
    Client baseClient = Client();
    return baseClient.send(req);
  }

  Future<PhraseApiResult> getTranslations(String distribution, String environment, String locale, String uuid, String? sdkVersion, String? lastUpdate, String? currentVersion, String? appVersion) async {
    Map<String, dynamic> params = Map<String, dynamic>();
    params['client'] = 'flutter';
    params['sdk_version'] = sdkVersion;
    params['unique_identifier'] = uuid;
    if(lastUpdate!=null) params['last_update'] = lastUpdate;
    if(currentVersion!=null) params['current_version'] = currentVersion;
    if(appVersion!=null) params['app_version'] = appVersion;

    var uri = Uri.https(_BASE_URL, '/$distribution/$environment/$locale/arb', params);
    String? version;
    while(true){
      StreamedResponse response = await _get(uri);
      int code = response.statusCode;
      if(code == 304){
        return PhraseApiResult.notModified();
      }else if(code>=300 && code<=399){
        uri = Uri.parse(response.headers['location']!);
        version = uri.queryParameters['version'];
        continue;
      }else if(code == 200){
        if(version==null) throw PhraseApiException("Missing version");
        var arb = await response.stream.bytesToString(utf8);
        return PhraseApiResult.success(version, arb);
      }else{
        throw PhraseApiException("Unexpected response code: $code");
      }
    }
  }

}